package net.sourceforge.nattable.command;

import net.sourceforge.nattable.layer.ILayer;
import net.sourceforge.nattable.layer.event.StructuralRefreshEvent;

/**
 * Command handler for handling {@link StructuralRefreshCommand}s.
 * Simply fires a {@link StructuralRefreshEvent}.
 * 
 * Needed to be able to refresh all layers by simply calling a command on the NatTable
 * instance itself (Remember that events are fired bottom up the layer stack while commands
 * are propagated top down). 
 * 
 * To refresh all layers by calling a {@link StructuralRefreshCommand} on the NatTable
 * instance, the {@link StructuralRefreshCommandHandler} should be registered against
 * the DataLayer.
 * 
 * @author fipro
 */
public class StructuralRefreshCommandHandler implements ILayerCommandHandler<StructuralRefreshCommand> {

	public Class<StructuralRefreshCommand> getCommandClass() {
		return StructuralRefreshCommand.class;
	}

	public boolean doCommand(ILayer targetLayer, StructuralRefreshCommand command) {
		targetLayer.fireLayerEvent(new StructuralRefreshEvent(targetLayer));
		return true;
	}

}
