package net.sourceforge.nattable.export;

import java.io.IOException;
import java.io.OutputStream;

import org.eclipse.swt.widgets.Shell;

import net.sourceforge.nattable.config.IConfigRegistry;
import net.sourceforge.nattable.layer.cell.LayerCell;
import net.sourceforge.nattable.style.ConfigAttribute;

public interface ILayerExporter {

	public static final ConfigAttribute<ILayerExporter> CONFIG_ATTRIBUTE = new ConfigAttribute<ILayerExporter>();

	OutputStream getOutputStream(Shell shell);
	
	/**
	 * Should be called only once at the beginning of an export operation. If the exporter supports exporting multiple layers as part of a single
	 * export operation, this method will be called only once before any layers are exported.
	 * @param outputStream
	 * @throws IOException
	 */
	void exportBegin(OutputStream outputStream) throws IOException;
	
	/**
	 * Should be called only once at the end of an export operation. If the exporter supports exporting multiple layers as part of a single
	 * export operation, this method will be called only once after all layers are exported.
	 * @param outputStream
	 * @throws IOException
	 */
	void exportEnd(OutputStream outputStream) throws IOException;
	
	void exportLayerBegin(OutputStream outputStream, String layerName) throws IOException;
	
	void exportLayerEnd(OutputStream outputStream, String layerName) throws IOException;
	
	void exportRowBegin(OutputStream outputStream, int rowPosition) throws IOException;
	
	void exportRowEnd(OutputStream outputStream, int rowPosition) throws IOException;
	
	void exportCell(OutputStream outputStream, Object exportDisplayValue, LayerCell cell, IConfigRegistry configRegistry) throws IOException;
	
}
