package net.sourceforge.nattable.group.command;

import java.util.ArrayList;
import java.util.List;

import net.sourceforge.nattable.command.AbstractLayerCommandHandler;
import net.sourceforge.nattable.group.RowGroupExpandCollapseLayer;
import net.sourceforge.nattable.group.RowGroupUtils;
import net.sourceforge.nattable.group.model.IRowGroup;
import net.sourceforge.nattable.group.model.IRowGroupModel;
import net.sourceforge.nattable.hideshow.event.HideRowPositionsEvent;
import net.sourceforge.nattable.hideshow.event.ShowRowPositionsEvent;
import net.sourceforge.nattable.layer.event.ILayerEvent;

public class RowGroupExpandCollapseCommandHandler<T> extends AbstractLayerCommandHandler<RowGroupExpandCollapseCommand> {

	private final RowGroupExpandCollapseLayer<T> rowGroupExpandCollapseLayer;

	public RowGroupExpandCollapseCommandHandler(RowGroupExpandCollapseLayer<T> rowGroupExpandCollapseLayer) {
		this.rowGroupExpandCollapseLayer = rowGroupExpandCollapseLayer;
	}
	
	public Class<RowGroupExpandCollapseCommand> getCommandClass() {
		return RowGroupExpandCollapseCommand.class;
	}

	@Override
	protected boolean doCommand(RowGroupExpandCollapseCommand command) {
		
		int rowIndex = rowGroupExpandCollapseLayer.getRowIndexByPosition(command.getRowPosition());
		IRowGroupModel<T> model = rowGroupExpandCollapseLayer.getModel();
		IRowGroup<T> group = RowGroupUtils.getTopMostParentGroup(RowGroupUtils.getRowGroupForRowIndex(model, rowIndex));
		
		// if group of rowIndex is not collapseable return without any 
		// further operation ...
		if (group == null || !group.isCollapseable()) {
			return true;
		}
		
		boolean wasCollapsed = group.isCollapsed();
		
		if (wasCollapsed) {
			group.expand();
		} else {
			group.collapse();
		}
		
		List<Integer> rowIndexes = new ArrayList<Integer>(RowGroupUtils.getRowIndexesInGroup(model, rowIndex));	
		List<Integer> rowPositions = RowGroupUtils.getRowPositionsInGroup(rowGroupExpandCollapseLayer, rowIndexes);
		
		ILayerEvent event;
		if (wasCollapsed) {
			event = new ShowRowPositionsEvent(rowGroupExpandCollapseLayer, rowPositions);
		} else {
			event = new HideRowPositionsEvent(rowGroupExpandCollapseLayer, rowPositions);
		}
		
		rowGroupExpandCollapseLayer.fireLayerEvent(event);
		
		return true;
	}
}
