package net.sourceforge.nattable.painter.cell;

import net.sourceforge.nattable.config.CellConfigAttributes;
import net.sourceforge.nattable.config.IConfigRegistry;
import net.sourceforge.nattable.data.convert.IDisplayConverter;
import net.sourceforge.nattable.layer.IUniqueIndexLayer;
import net.sourceforge.nattable.layer.LayerUtil;
import net.sourceforge.nattable.layer.cell.LayerCell;
import net.sourceforge.nattable.util.GUIHelper;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;

public class ColumnHeaderCheckBoxPainter extends ImagePainter {
	
	private static final Log log = LogFactory.getLog(ColumnHeaderCheckBoxPainter.class);

	private final Image checkedImg;
	private final Image semicheckedImg;
	private final Image uncheckedImg;
	
	private final IUniqueIndexLayer columnDataLayer;

	public ColumnHeaderCheckBoxPainter(IUniqueIndexLayer columnDataLayer) {
		this(
				columnDataLayer,
				GUIHelper.getImage("checked"),
				GUIHelper.getImage("semichecked"),
				GUIHelper.getImage("unchecked")
		);
	}

	public ColumnHeaderCheckBoxPainter(IUniqueIndexLayer columnLayer, Image checkedImg, Image semicheckedImage, Image uncheckedImg) {
		this.columnDataLayer = columnLayer;
		this.checkedImg = checkedImg;
		this.semicheckedImg = semicheckedImage;
		this.uncheckedImg = uncheckedImg;
	}

	public int getPreferredWidth(boolean checked) {
		return checked ? checkedImg.getBounds().width : uncheckedImg.getBounds().width;
	}

	public int getPreferredHeight(boolean checked) {
		return checked ? checkedImg.getBounds().height : uncheckedImg.getBounds().height;
	}

	public void paintIconImage(GC gc, Rectangle rectangle, int yOffset, boolean checked) {
		Image checkBoxImage = checked ? checkedImg : uncheckedImg;

		// Center image
		int x = rectangle.x + (rectangle.width / 2) - (checkBoxImage.getBounds().width/2);

		gc.drawImage(checkBoxImage, x, rectangle.y + yOffset);
	}

	@Override
	protected Image getImage(LayerCell cell, IConfigRegistry configRegistry) {
		int columnPosition = LayerUtil.convertColumnPosition(cell.getLayer(), cell.getColumnPosition(), columnDataLayer);
		
		int checkedCellsCount = getCheckedCellsCount(columnPosition, configRegistry);
		
		if (checkedCellsCount > 0) {
			if (checkedCellsCount == columnDataLayer.getRowCount()) {
				return checkedImg;
			} else {
				return semicheckedImg;
			}
		} else {
			return uncheckedImg;
		}
	}

	public int getCheckedCellsCount(int columnPosition, IConfigRegistry configRegistry) {
		int checkedCellsCount = 0;
		
		for (int rowPosition = 0; rowPosition < columnDataLayer.getRowCount(); rowPosition++) {
			LayerCell columnCell = columnDataLayer.getCellByPosition(columnPosition, rowPosition);
			if (isChecked(columnCell, configRegistry)) {
				checkedCellsCount++;
			}
		}
		return checkedCellsCount;
	}

	protected boolean isChecked(LayerCell cell, IConfigRegistry configRegistry) {
		return convertDataType(cell, configRegistry).booleanValue();
	}

	protected Boolean convertDataType(LayerCell cell, IConfigRegistry configRegistry) {
		if (cell.getDataValue() instanceof Boolean) {
			return (Boolean) cell.getDataValue();
		}
		IDisplayConverter displayConverter = configRegistry.getConfigAttribute(CellConfigAttributes.DISPLAY_CONVERTER, cell.getDisplayMode(), cell.getConfigLabels().getLabels());
		Boolean convertedValue = null;
		if (displayConverter != null) {
			try {
				convertedValue = (Boolean) displayConverter.canonicalToDisplayValue(cell, configRegistry, cell.getDataValue());
			} catch (Exception e) {
				log.debug(e);
			}
		}
		if (convertedValue == null) {
			convertedValue = Boolean.FALSE;
		}
		return convertedValue;
	}
	
}
