package net.sourceforge.nattable.selection;

import java.util.List;
import java.util.Set;

import net.sourceforge.nattable.coordinate.Range;

import org.eclipse.swt.graphics.Rectangle;

/**
 * Tracks the selections made in the table.
 */
public interface ISelectionModel {

	public void addSelection(int columnPosition, int rowPosition);

	public void addSelection(final Rectangle range);

	public void clearSelection();

	public void clearSelection(int columnPosition, int rowPosition);

	public void clearSelection(Rectangle removedSelection);

	public boolean isEmpty();

	public List<Rectangle> getSelections();
	
	// Cell features

	public boolean isCellPositionSelected(int columnPosition, int rowPosition);
	
	// Column features

	public int[] getSelectedColumnPositions();

	public boolean isColumnPositionSelected(int columnPosition);

	/**
	 * @param columnHeight the number of rows in a fully selected column
	 */
	public int[] getFullySelectedColumnPositions(int columnHeight);

	/**
	 * @param columnHeight the number of rows in a fully selected column
	 */
	public boolean isColumnPositionFullySelected(int columnPosition, int columnHeight);

	// Row features

	public int getSelectedRowCount();
	
	public Set<Range> getSelectedRowPositions();
	
	public boolean isRowPositionSelected(int rowPosition);

	/**
	 * @param rowWidth the number of columns in a fully selected row
	 */
	public int[] getFullySelectedRowPositions(int rowWidth);

	/**
	 * @param columnHeight the number of rows in a fully selected column
	 */
	public boolean isRowPositionFullySelected(int rowPosition, int rowWidth);
	
}
